const std = @import("std");
const fs = std.fs;
const os = std.os;
const posix = std.posix;

const ansi = @import("ansi.zig");
const Editor = @import("editor.zig").Editor;

fn uncook(handle: anytype, tcattr: anytype) !void {
    var raw_attr = tcattr;
    raw_attr.lflag.ECHO = false;
    raw_attr.lflag.ICANON = false;
    raw_attr.lflag.ISIG = false;
    raw_attr.lflag.IEXTEN = false;
    raw_attr.iflag.IXON = false;
    raw_attr.iflag.ICRNL = false;
    raw_attr.iflag.BRKINT = false;
    raw_attr.iflag.INPCK = false;
    raw_attr.iflag.ISTRIP = false;
    raw_attr.oflag.OPOST = false;
    raw_attr.cflag.CSIZE = .CS8;
    // TODO think about these
    raw_attr.cc[@intFromEnum(std.posix.V.TIME)] = 0;
    raw_attr.cc[@intFromEnum(std.posix.V.MIN)] = 1;
    try std.posix.tcsetattr(handle, .FLUSH, raw_attr);
}

fn run() !void {
    var tty = try fs.cwd().openFile("/dev/tty", .{ .mode = .read_write });
    defer tty.close();

    const cooked_attr = try std.posix.tcgetattr(tty.handle);
    defer std.posix.tcsetattr(tty.handle, .FLUSH, cooked_attr) catch {};
    try uncook(tty.handle, cooked_attr);

    const stdout_file = std.io.getStdOut().writer();
    var bw = std.io.bufferedWriter(stdout_file);
    defer bw.flush() catch {};
    const stdout = bw.writer();
    const writeCsi = ansi.writeCsi;

    // Hide / unhide cursor
    try writeCsi(stdout, "?25l");
    defer writeCsi(stdout, "?25h") catch {};
    // Mouse on / off
    // try writeCsi(stdout, "?1000h");
    // defer writeCsi(stdout, "?1000l") catch {};
    // Save / restore cursor pos
    try writeCsi(stdout, "s");
    defer writeCsi(stdout, "u") catch {};
    // Save / restore screen
    try writeCsi(stdout, "?47h");
    defer writeCsi(stdout, "?47l") catch {};
    // Enable alternative buffer / switch back to original buffer
    try writeCsi(stdout, "?1049h");
    defer writeCsi(stdout, "?1049l") catch {};
    try bw.flush();

    var editor = try Editor.init(tty, &bw);
    defer editor.deinit();
    var args = std.process.args();
    if (args.next()) |_| {
        if (args.next()) |path| {
            if (args.next() == null)
                try editor.open(path);
        }
    }
    try editor.run();
}

pub fn main() !void {
    return run() catch |e| switch (e) {
        error.OutOfMemory => std.io.getStdOut().writer().writeAll(@import("e/oom.zig").oom),
        else => e,
    };
}
